pub struct RepliconSharedPlugin {
pub auth_method: AuthMethod,
}Expand description
Initializes types, resources and events needed for both client and server.
Fields
auth_method: AuthMethodConfigures the authorization process.
Examples
Custom authorization to set a player name before starting replication.
We re-use ProtocolMismatch, which is registered only with
AuthMethod::ProtocolCheck, but it could be any event.
use bevy::{prelude::*, state::app::StatesPlugin};
use bevy_replicon::prelude::*;
use serde::{Deserialize, Serialize};
let mut app = App::new();
app.add_plugins((
MinimalPlugins,
StatesPlugin,
RepliconPlugins.set(RepliconSharedPlugin {
auth_method: AuthMethod::Custom,
}),
))
.add_client_event::<ClientInfo>(Channel::Ordered)
.add_server_event::<ProtocolMismatch>(Channel::Unreliable)
.make_event_independent::<ProtocolMismatch>() // Let the client receive it without replication.
.add_observer(start_game)
.add_systems(OnEnter(ClientState::Connected), send_info);
fn send_info(
mut commands: Commands,
protocol: Res<ProtocolHash>,
) {
commands.client_trigger(ClientInfo {
protocol: *protocol,
player_name: "Shatur".to_string(), // Could be read from console or UI.
});
}
fn start_game(
client_info: On<FromClient<ClientInfo>>,
mut commands: Commands,
mut disconnects: MessageWriter<DisconnectRequest>,
protocol: Res<ProtocolHash>,
) {
let client = client_info
.client_id
.entity()
.expect("protocol hash sent only from clients");
// Since we are using custom authorization,
// we need to verify the protocol manually.
if client_info.protocol != *protocol {
// Notify the client about the problem. No delivery
// guarantee, since we disconnect after sending.
commands.server_trigger(ToClients {
targets: SendTargets::Single(client_info.client_id),
message: ProtocolMismatch,
});
disconnects.write(DisconnectRequest { client });
}
// Validate player name, run the necessary game logic...
// Manually mark the client as authorized.
commands.entity(client).insert(AuthorizedClient);
}
/// A client event for custom authorization.
#[derive(Event, Serialize, Deserialize)]
struct ClientInfo {
protocol: ProtocolHash,
player_name: String,
}Trait Implementations
Sourcefn default() -> RepliconSharedPlugin
fn default() -> RepliconSharedPlugin
Returns the “default value” for a type. Read more
Sourcefn finish(&self, app: &mut App)
fn finish(&self, app: &mut App)
Finish adding this plugin to the
App, once all plugins registered are ready. This can
be useful for plugins that depends on another plugin asynchronous setup, like the renderer.Sourcefn ready(&self, _app: &App) -> bool
fn ready(&self, _app: &App) -> bool
Has the plugin finished its setup? This can be useful for plugins that need something
asynchronous to happen before they can finish their setup, like the initialization of a renderer.
Once the plugin is ready,
finish should be called.Sourcefn cleanup(&self, _app: &mut App)
fn cleanup(&self, _app: &mut App)
Runs after all plugins are built and finished, but before the app schedule is executed.
This can be useful if you have some resource that other plugins need during their build step,
but after build you want to remove it and send it to another thread.